//
//  AASFSCopyObjectHandler.h
//  AASFilesystemTools
//
/* Copyright (c) 2001-2009, Andreas Schwarz
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
 
 Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
 Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
 Neither the names of the Andreas Schwarz or immaterial nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
 
 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#import <Cocoa/Cocoa.h>


#pragma mark Keys


/* The error domain for errors returned by these objects. */
extern NSString *AASFSCopyObjectErrorDomain;

/* Errors for AASFSCopyObjectErrorDomain */
typedef enum {
	AASFSCOErrorUnknown						= -99,	// Unknown error.
	AASFSCOErrorUserCanceled				= -1,	// User cancelled operation.
	AASFSCOErrorCannotGetFSRef				= -101,	// Operation was unable to get FSRef for the given URL.
	AASFSCOErrorCannotGetPathRep			= -102,	// Operation was unable to get filesystem representation for the given URL.
	AASFSCOErrorCannotScheduleOp			= -103,	// Unable to schedule the file operation with run loop.
	AASFSCOErrorBeginningOp					= -104,	// Error beginning file operation.
	AASFSCOErrorUnspecifiedOperation		= -105,	// The operation specified was unknown.
	AASFSCOErrorCannotCreateOperation		= -106,	// Could not create the file operation object.
	AASFSCOErrorGettingDeviceNum			= -107,	// Could not get device number for file.
	AASFSCOErrorGettingPreflightInfo		= -108,	// Error getting file information during preflight.
	AASFSCOErrorIteratingFiles				= -109,	// Error iterating through files during preflight.
	AASFSCOErrorProcessingFiles				= -110	// Error during the actual move/copy operation thread.
} AASFSCOError;





/* Keys used in dictionaries that are passed to the delegate. */
extern NSString *AASFSCopyObjectErrorKey;					// Key for the NSError object describing the error.
extern NSString *AASFSCopyObjectHandlerKey;					// Key for the AASFSCopyObjectHandler instance.
extern NSString *AASFSCopyObjectURLKey;						// Key for an NSURL to the file being copied/moved.
extern NSString *AASFSCopyObjectSourceKey;					// Key for an NSURL to the source file (the file in the initial array whose descendant is copying).
extern NSString *AASFSCopyObjectOperationStatusKey;			// Key for the current status of the operation.
extern NSString *AASFSCopyObjectPercentCompleteKey;			// Key for the precentage of the operation that has completed.




/* Operation types */
typedef enum {
	AASFSCopyObjectCopyOperation	= 1,		// Denotes a file copy/duplicate operation
	AASFSCopyObjectMoveOperation	= 2,		// Denotes a file move operation
	AASFSCopyObjectTrashOperation	= 3,		// Denotes a move to trash operation
} AASFSCopyObjectOperationType;


/* Operation status values */
typedef enum {
	AASFSCopyObjectStatusPreparing	= 1,		// Denotes an operation which is waiting to begin
	AASFSCopyObjectStatusPreflight	= 2,		// Denotes preflight checking which occurs before some operations
	AASFSCopyObjectStatusOperating	= 3,		// Denotes operation is in progress
	AASFSCopyObjectStatusCompleted	= 4,		// Denotes an operation which is complete
	AASFSCopyObjectStatusCancelled	= 5			// Denotes an operation which was canceled
} AASFSCopyObjectStatus;




#pragma	mark -
#pragma	mark Utility Functions

BOOL FSRefFromURL(CFURLRef url, FSRef *newRef, NSError **error);
NSError * BuildError(AASFSCOError errorCode, NSError *underlyingError, NSURL *url);




#pragma	mark -

@class AASFSCopyObjectPreflightOperation;

@interface AASFSCopyObjectHandler : NSObject {
	AASFSCopyObjectPreflightOperation *preflightOperation;
	NSMutableDictionary *defaultDictionary;
	NSArray *urls;
	NSURL *destination;
	NSURL *currentSource;
	NSOperationQueue *operationQueue;
	
	id delegate;
	SEL updateSelector;
	SEL newNameforDuplicateSelector;
	
	AASFSCopyObjectOperationType operationType;
	AASFSCopyObjectStatus status;
	
	BOOL overwrite;
	BOOL moveAcrossVolumes;
	
	BOOL cancelled;
	BOOL preflightFinished;
	NSUInteger urlsArrayIndex;
	FSFileOperationRef fileOperation;
	
	unsigned long long totalBytes;
	unsigned long long currentBytes;
	NSUInteger totalFiles;
	NSUInteger currentFiles;
}


#pragma	mark -
#pragma mark Required Settings

/* Delegate that wants to recieve status updates (sent on main thread) and selector the delegate wants us to use. Selector must take one argument, which will be an NSDictionary containing the usual kFSOperation... keys/values, AASFSCopyObjectOperationStatusKey, and AASFSCopyObjectHandlerKey, plus maybe a few extra keys described elsewhere. */
@property(assign) id delegate;
@property(assign) SEL updateSelector;

/* The array of NSURL objects which you wish to operate on, and the destination to which you want to move/copy them. Destination is not required for "move to trash" operations. */
@property(copy) NSArray *urls;
@property(copy) NSURL *destination;

/* The operation queue you want to use for the operation. */
@property(retain) NSOperationQueue *operationQueue;




#pragma	mark -
#pragma mark Other Options

/* Optional. If set, this selector will be used to ask the delegate for a new name for a file during a duplication operation. Duplicating a file means it is being "copied" into the same folder where it already resides. If the selector is not set and this situation arises, the file is not duplicated. The selector should take one object (which will be a dictionary with AASFSCopyObjectHandlerKey and AASFSCopyObjectURLKey values set) and should return an NSString that represents a unique filename in the url's parent folder. Check to ensure it is unique! A sample method is provided at the bottom of this header file. */
@property(assign) SEL newNameforDuplicateSelector;

/* Set whether the copied or moved files should overwrite preexisting files at the destination. Defaults to NO. */
@property(assign) BOOL overwrite;

/* Set whether moves should be allowed across volumes. Defaults to YES. */
@property(assign) BOOL moveAcrossVolumes;

/* The type of operation you want to do. Defaults to AASFSCopyObjectCopyOperation. */
@property(assign) AASFSCopyObjectOperationType operationType;



#pragma	mark -
#pragma mark Actions


/* Once you have set the above properties, start the operation using the start method. It is not safe to change the above properties after you've started the operation. Cancel will cancel the operation. */

- (void)start;
- (void)cancel;
@property(readonly, assign, getter=isCancelled) BOOL cancelled;

@end




#pragma	mark -
#pragma mark Sample Method


/* Sample method for newNameforDuplicateSelector */

/*

- (NSString *)nameForDuplicateFile:(NSDictionary *)dict {
	NSURL *url = [dict objectForKey:AASFSCopyObjectURLKey];
	NSString *filePath = [url path];
	
	NSString *base = [filePath stringByDeletingPathExtension];
	NSString *extension = [filePath pathExtension];
	NSString *copyText = NSLocalizedString(@"Copy", @"");
	NSString *temp = nil;
	
	NSString *finalFilename;
	
	BOOL ext = ([extension length] > 0);
	NSUInteger a = 1;
	
	if (ext) {
		temp = [[NSString alloc] initWithFormat:@"%@ %@.%@", base, copyText, extension];
	} else {
		temp = [[NSString alloc] initWithFormat:@"%@ %@", base, copyText];
	}
	
	if ([temp length] > 255) {
		[temp release];
		
		// You might want to do something more user-friendly than this!
		base = [[base stringByDeletingLastPathComponent] stringByAppendingPathComponent:@"Duplicated File"];
		
		if (ext) {
			temp = [[NSString alloc] initWithFormat:@"%@ %@.%@", base, copyText, extension];
		} else {
			temp = [[NSString alloc] initWithFormat:@"%@ %@", base, copyText];
		}
	}
	
	while ([[NSFileManager defaultManager] fileExistsAtPath:temp]) {
		
		[temp release];
		
		if (ext) {
			temp = [[NSString alloc] initWithFormat:@"%@ %@ %d.%@", base, copyText, a, extension];
		} else {
			temp = [[NSString alloc] initWithFormat:@"%@ %@ %d", base, copyText, a];
		}
		
		a++;
	}
	
	finalFilename = [temp lastPathComponent];
	[temp release];
	
    return finalFilename;
}

*/












